﻿using System;
using System.Text.RegularExpressions;
using System.Windows;
using System.Windows.Input;

namespace SearchPath
{
    public partial class MainWindow : Window
    {
        Map map;
        Astar astar;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MenuItem_Start(object sender, RoutedEventArgs e)
        {
            int fromCell = Convert.ToInt32(tbFrom.Text);
            int toCell = Convert.ToInt32(tbTo.Text);
            if (fromCell < 0 || fromCell > 255 || toCell < 0 || toCell > 255)
            {
                MessageBox.Show("Only numbers between 0 and 255 allowed.");
                return;
            }
            map = new Map(canvasPlot, "..\\..\\map.txt");
            astar = new Astar(map, lvOpenList, lvClosedList, fromCell, toCell);
            astar.Init();
            map.DrawPathSegment(lvClosedList);
        }

        private void canvasArea_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (map == null || astar == null) return;
            astar.SingleStep();
            map.DrawPathSegment(lvClosedList);
        }

        private void tbFromTextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            if (Regex.IsMatch(tbFrom.Text, "[^0-9]"))
            {
                MessageBox.Show("Please enter only numbers between 0 and 255.");
                tbFrom.Text.Remove(tbFrom.Text.Length - 1);
            }
        }

        private void tbToTextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            if (Regex.IsMatch(tbTo.Text, "[^0-9]"))
            {
                MessageBox.Show("Please enter only numbers.");
                tbTo.Text.Remove(tbTo.Text.Length - 1);
            }
        }
    }
}
