﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media;
using System.IO;

namespace SearchPath
{
    public class Map
    {
        Canvas canvasPlot;
        double cellHeight;
        double cellWidth;

        int maxColumns;
        public int MaxColumns { get { return maxColumns; } }
        int maxRows;
        public int MaxRows { get { return maxRows; } }
        
        public Cell[] arrayCells;

        public Map(Canvas canvasPlot, string fileName)
        {
            this.canvasPlot = canvasPlot;
            String strMap = ReadMapFile(fileName);
            cellHeight = canvasPlot.ActualHeight / maxRows;
            cellWidth = canvasPlot.ActualWidth / maxColumns;
            BuildMap(strMap);

            DrawMap();
        }

        private void BuildMap(String strMap)
        {
            arrayCells = new Cell[maxColumns * maxRows];
            for (int idx = 0; idx < arrayCells.Length; idx++)
            {
                char c = strMap[idx];
                CellType ct = (CellType)Convert.ToInt32(c.ToString());
                arrayCells[idx] = new Cell(ct, idx);
            }
        }

        private string ReadMapFile(string FileName)
        {
            
            string str;
            using (StreamReader sr = new StreamReader(FileName))
            {
                str = sr.ReadLine();
                maxColumns = Convert.ToInt32(str);
                str = sr.ReadLine();
                maxRows = Convert.ToInt32(str);
                StringBuilder stringBuilder = new StringBuilder(maxColumns * maxRows);  
                while (!sr.EndOfStream)
                {
                    stringBuilder.Append(sr.ReadLine());
                }
                return stringBuilder.ToString();
            }
        }

        public int GetFromToCost(int actNodeIdx, int nextNodeIdx, bool orthogonal)
        {
            return (int) (orthogonal ? 
                0.5 * (arrayCells[nextNodeIdx].cost + arrayCells[actNodeIdx].cost) :
                0.7 * (arrayCells[nextNodeIdx].cost + arrayCells[actNodeIdx].cost)) ;
        }

        public void DrawPathSegment(ListView lvClosedList)
        {
            Line line;

            if (lvClosedList.Items.Count > 1)
            {
                int idx = ((Node)lvClosedList.Items[lvClosedList.Items.Count - 1]).MyIdx;
                int fromIdx = ((Node)lvClosedList.Items[lvClosedList.Items.Count - 1]).FromIdx;
                line = new Line();
                line.X1 = (idx % maxColumns) * cellWidth + cellWidth / 2;
                line.X2 = (fromIdx % maxColumns) * cellWidth + cellWidth / 2;
                line.Y1 = (idx / maxColumns) * cellHeight + cellHeight / 2;
                line.Y2 = (fromIdx / maxColumns) * cellHeight + cellHeight / 2; ;
                line.StrokeThickness = 1;
                line.Stroke = new SolidColorBrush(Colors.Red);
                canvasPlot.Children.Add(line);
            }
        }

        private void DrawMap()
        {
            canvasPlot.Children.Clear();
            DrawCellContent();
            DrawCellBorder();
        }

        private void DrawCellContent()
        {
            TextBlock tb;
            int cellIdx = 0;
            for (int rows = 0; rows < maxRows; rows++)
            {
                for (int columns = 0; columns < maxColumns; columns++)
                {
                    Rectangle rect = new Rectangle();
                    rect.Height = cellHeight;
                    rect.Width = cellWidth;
                    rect.Fill = new SolidColorBrush(arrayCells[rows * maxColumns + columns].color);

                    canvasPlot.Children.Add(rect);
                    Canvas.SetLeft(rect, columns * cellWidth);
                    Canvas.SetTop(rect, rows * cellHeight);

                    tb = new TextBlock();
                    tb.Text = cellIdx.ToString();
                    canvasPlot.Children.Add(tb);
                    Canvas.SetLeft(tb, columns * cellWidth + cellWidth / 2);
                    Canvas.SetTop(tb, rows * cellHeight + cellHeight / 2);
                    cellIdx++;
                }
            }
        }

        private void DrawCellBorder()
        {
            Line line;
            for (int rows = 0; rows <= maxRows; rows++)
            {
                line = new Line();
                line.X1 = 0;
                line.X2 = canvasPlot.ActualWidth;
                line.Y1 = line.Y2 = rows * cellHeight;
                line.Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0));
                canvasPlot.Children.Add(line);
            }
            for (int columns = 0; columns <= maxColumns; columns++)
            {
                line = new Line();
                line.X1 = line.X2 = columns * cellWidth;
                line.Y1 = 0;
                line.Y2 = canvasPlot.ActualHeight;
                line.Stroke = new SolidColorBrush(Color.FromRgb(0, 0, 0));
                canvasPlot.Children.Add(line);
            }
        }

        public void DrawPath(int startCellIdx, List<Node> ClosedItemSource)
        {
            Line line;

            if (ClosedItemSource.Count > 1)
            {
                int idx = ((Node)ClosedItemSource[ClosedItemSource.Count - 1]).MyIdx;
                int fromIdx = ((Node)ClosedItemSource[ClosedItemSource.Count - 1]).FromIdx;
                do
                {
                    line = new Line();
                    line.X1 = (idx % MaxColumns) * cellWidth + cellWidth / 2;
                    line.X2 = (fromIdx % MaxColumns) * cellWidth + cellWidth / 2;
                    line.Y1 = (idx / MaxColumns) * cellHeight + cellHeight / 2;
                    line.Y2 = (fromIdx / MaxColumns) * cellHeight + cellHeight / 2; ;
                    line.StrokeThickness = 5;
                    line.Stroke = new SolidColorBrush(Color.FromRgb(255, 0, 0));
                    canvasPlot.Children.Add(line);
                    Node node = ClosedItemSource.Find(item => item.MyIdx == fromIdx);
                    idx = node.MyIdx;
                    fromIdx = node.FromIdx;
                }
                while (idx != startCellIdx);
            }
        }
    }
}
